! SUBROUTINE CMPTBL ====*====3====*====4====*====5====*====6====*====7
!
! -- input --
!
!   JD = 0 ... single group
!      = 1 ... double group
!
!   ( KBB(1),KBB(2),KBB(3) ) / ICC  : reciprocal lattice
!   NRT : Number of Ireducible Representation
!
!   NH  : Number of Elements of K-point Group
!   MTR : Number of Time Reversal Element
!
!---*----1----*----2----*----3----*----4----*----5----*----6----*----7
      SUBROUTINE CMPTBL(JD,KBB,ICC,NRT,NH,MTR)
      use scl, only:ipr
      use axp, only:ifkp,kkb,iccb,icptbl,indm,npoint
      use enr, only:kxm,icm,mrnm
      implicit none
!
      integer :: JD,KBB(3),ICC,NRT,NH,MTR
!
      integer :: KB(3),IC
      integer :: KBX(3),ICX

      integer :: JDOB,NRTT,INDC
      integer :: ITCR(12),IND
      integer :: NRR,NHH,NSTR, MTRR,ND(12), JTR(12), IPA(12)
      integer :: NRX,NHX,NSTRX,MTRX,NDX(12),JTRX(12),IPAX(12)
      integer :: ICP(12,12),ICP0(12,12)
      integer :: IR1,IR,IW1
      integer :: i,j,k
!
      JDOB=JD
      do I=1,NPOINT
         INDM(I)=0
      enddo

      DO 1 I=1,NPOINT
         IF(INDM(I).NE.0) cycle
         do j=1,3
           KB(j)=KKB(j,I)
           KBX(j)=KXM(j,IFKP(I))
         enddo
         IC=ICCB(I)
         ICX=ICM(IFKP(I))

         !-------------------------------------------------------------
         !
         ! == Specification Subroutine CORRES(in TSPACE) ===
         !
         !     CALL CORRES(KB,IC,KBX,ICX,JDOB,ITCR,NRR,IND)
         !
         !   When KB(3)/IC is equivalent to KBX(3)/ICX, you obtain
         !  correspondence table. ITCR(12) is given by the calculation for
         !  the character of FULL representation of space group.
         !
         ! -- INPUT --
         !
         !    KB(3) / IC
         !    KBX(3) /ICX
         !
         !    JDOB = 0 ... single representation 
         !    JDOB = 1 ... double representation 
         !
         ! -- OUTPUT --
         !
         !    NRR:Number of Representation
         !    ICTR(IR) : Table of Correspondence for IR (Irreducible  Representation)
         !    IND =  1 ... ITCR(IR) for KB is the same as IR for KBB
         !    IND = -1 ... ITCR(IR) for KB is the time reversal of IR for KBX
         !    IND =  0 ... No correspondence.
         !
         !-------------------------------------------------------------
         CALL CORRES(KB,IC,KBX,ICX,JDOB,ITCR,NRR,IND)

         IF(IND.EQ.0) THEN
            write(6,*) ' kb=',kb,ic,' kbx=',kbx,icx
            WRITE(6,*) ' STOP AT CORRES IN CMPTBL'
            STOP
         END IF

         !====================================================================================
         !
         ! for ( KBX(1), KBX(2), KBX(3) ) / ICX
         !
         !====================================================================================

         !-------------------------------------------------------------
         !
         ! obtain small representation
         !
         !-------------------------------------------------------------
         CALL TSIREP(KBX,ICX,JDOB)

         !-------------------------------------------------------------
         !
         ! == Specification Subroutine DGTRMD(in TSPACE) ===
         !
         !      CALL DGTRMD(JD,NR,NH,NSTR,ND,MTR,JTR,IPA)
         !
         ! -- input --
         !  JD=0 (single group)
         !  JD=1 (double group)
         !
         ! -- output --
         !  NR     : Number of Irreducible Representation
         !  NH     : Number of Elements of K-point Group
         !  NSTR   : Number of Star
         !  ND(12) : Deneracy of Irreducible Represenation
         !  MTR    : Number of Time Reversal Element
         !  JTR(12): Herring Sum
         !  IPA(12): Partner
         !
         !
         ! modified by H.Funashima,2011
         !   bug fixed by H.Funashima, 2011: JDO -> JD call sub. DGTRMD
         !
         !-------------------------------------------------------------
         CALL DGTRMD(JD,NRX,NHX,NSTRX,NDX,MTRX,JTRX,IPAX)



         !====================================================================================
         !
         ! for ( KB(1), KB(2), KB(3) ) / IC
         !
         !====================================================================================

         CALL TSIREP(KB,IC,JDOB)
         CALL DGTRMD(JD,NRR,NHH,NSTR,ND,MTRR,JTR,IPA)

         do IR1=1,NRX
            IF( JTR(IR1).EQ.0  .AND.  IR1.LT.IPAX(IR1)  .AND.  ITCR(IR1).GT.IPA(ITCR(IR1))  ) THEN
                 IW1=ITCR(IR1)
                 ITCR(IR1)=ITCR(IPAX(IR1))
                 ITCR(IPAX(IR1))=IW1
            END IF
         enddo


         IF( NH.NE.NHH  .OR.  MTR.NE.MTRR ) THEN

            !-------------------------------------------------------------
            !
            ! == Specification Subroutine COMPAT(in TSPACE) ===
            !
            !      CALL COMPAT(KBB,ICC,NRTT,KB,IC,NRR,ICP,JDOB,INDC)
            ! 
            !   Compatibility of the representation of KBB(3)/ICC
            !     to that of KB(3)/IC, when k-point group of KB is a subgroup
            !
            ! -- input --
            !
            !  NRTT: Number of Ireducible Representation for KBB/ICC 
            !  NRR : Number of Ireducible Representation for KB/IC 
            !  
            !  JDOB = 0 ... single representation 
            !  JDOB = 1 ... double representation 
            !
            ! -- output --
            !
            !  ICP(1-NRTT,1-NRR): table of compatibility 
            !  INDC = 0 ... normal end
            !       = 1 ... error
            !-------------------------------------------------------------
            CALL COMPAT(KBB,ICC,NRTT,KB,IC,NRR,JDOB,ICP,INDC)

            IF(INDC.NE.0) THEN
               WRITE(6,*) 'STOP AT COMPAT IN COMTBL'
               STOP
            END IF



            !-------------------------------------------------------------
            !
            ! == Specification Subroutine CMPTRV(in TSPACE) ===
            !
            !      CALL CMPTRV(KBB,ICC,KB,IC,JDOB,ICP,ICP0)
            ! 
            !   Compatibility table ICP obtained by COMPAT is reformed to
            !     ICP0 by taking acoount the time reversal symmetry.
            !
            !
            ! -- input --
            !
            !   ICP : compatibility table ( no time reversal symmetry )
            !
            !  JDOB = 0 ... single representation 
            !  JDOB = 1 ... double representation 
            !
            !
            ! -- out put --
            !
            !   ICP0 : compatibility table ( time reversal symmetry )
            !-------------------------------------------------------------
            CALL CMPTRV(KBB,ICC,KB,IC,JDOB,ICP,ICP0)

            do IR=1,NRT
               ICPTBL(IR,I)=ICP0(ITCR(MRNM(IFKP(I))),IR)
            enddo

            INDM(I)=1

            IF(IPR.GE.4) then
                WRITE(6,'(14I3)') I,INDM(I),(ICPTBL(K,I),K=1,NRT)
            endif

            IF(I.EQ.NPOINT) cycle
            
            do J=I+1,NPOINT
               do K=1,3
                  IF(KB(K)*ICCB(J).NE.KKB(K,J)*IC) GO TO 1
               enddo
               do IR=1,NRT
                  ICPTBL(IR,J)=ICP0(ITCR(MRNM(IFKP(J))),IR)
               enddo
               INDM(J)=1
               IF(IPR.GE.4) then
                   WRITE(6,'(14I3)') J,INDM(J),(ICPTBL(K,J),K=1,NRT)
               endif
            enddo

         ELSE
            do IR=1,NRT
               ICPTBL(IR,I)=0
            enddo
            ICPTBL(ITCR(MRNM(IFKP(I))),I)=1
            INDM(I)=-1
            IF(IPR.GE.4) then
                WRITE(6,'(14I3)') I,INDM(I),(ICPTBL(K,I),K=1,NRT)
            endif
            IF(I.ne.NPOINT) then
                do J=I+1,NPOINT
                  do K=1,3
                    IF(KB(K)*ICCB(J).NE.KKB(K,J)*IC) GO TO 1
                  enddo
                  do IR=1,NRT
                    ICPTBL(IR,J)=0
                  enddo
                  ICPTBL(ITCR(MRNM(IFKP(J))),J)=1
                  INDM(J)=-1
                  IF(IPR.GE.4) then
                      WRITE(6,'(14I3)') J,INDM(J),(ICPTBL(K,J),K=1,NRT)
                  endif
                enddo
            endif
         END IF

    1 CONTINUE
      RETURN
      end
