
! module for ayband
!    written by Hiroki Funahsima, Tokyo Univ of Sci. and Osaka Univ.
!
!
module ayband_param
  implicit none
  integer :: maxkpt
  integer :: maxfkp
  integer :: maxirp
  integer :: maxeig
  integer,parameter :: N65=(2**7) + 1 ! 2**7 + 1 = 129 : Number of Iterpolation point
end module ayband_param

module scl
  implicit none
  real(8) :: wx,wy
  real(8) :: EO,EM
  real(8) :: XM,YM
  real(8) :: IPR,ISO
  integer :: nka
end module scl

module spgspl
  implicit none
  integer :: NIRG(12), NDIRG(12), NNRG
end module


module isp
  implicit none
  !==================================================================
  !
  !  KM(itype,Iaxis) : Name of K-Point
  !
  !     itype = 1 ... starting point of axis
  !     itype = 2 ... middle point of axis
  !     itype = 3 ... end point of axis
  !     
  !     Iaxis: Index of axis
  !
  !
  integer,allocatable :: IX(:),ICON(:)
  character(2),allocatable :: MK(:,:)
  real(8),allocatable :: RT(:),PS(:,:)
  integer :: naxm  ! number of axis
  !
  !==================================================================
  contains

  subroutine set_allocate_isp
    implicit none
    allocate(IX(naxm),RT(naxm),PS(2,naxm),ICON(naxm),MK(3,naxm))
    return
  end subroutine set_allocate_isp

end module isp


module jmpax
   implicit none
   integer,allocatable  :: INDJAX(:)
   real(8),allocatable  :: RTJMP(:)
   integer :: JOPT
   contains

   subroutine set_allocate_jmpax
     use isp, only:naxm
     implicit none
     allocate(INDJAX(naxm),RTJMP(naxm))
     return
   end subroutine set_allocate_jmpax
end module jmpax



module kkend
  !
  ! change, 2011 by H.Funashima
  !
  ! old ayband kk1m(3,naxm) -> kkm(1,3,naxm)
  !            kk2m(3,naxm) -> kkm(2,3,naxm)
  !            icc1m(naxm)  -> iccm(1,naxm)
  !            icc2m(naxm)  -> iccm(2,naxm)
  !
  implicit none
  integer,allocatable :: kkm(:,:,:)
  integer,allocatable :: iccm(:,:)
  contains

  subroutine set_allocate_kkend
    use isp, only:naxm
    implicit none
    allocate(kkm(2,3,naxm) )
    allocate(iccm(2,naxm) )
    return
  end subroutine set_allocate_kkend

end module kkend


module axe
  implicit none
  real(8),allocatable :: eax(:,:,:)
  integer,allocatable :: NLIN(:),JRR(:),JUD(:)
  integer :: naxen
end module axe

module axp
  integer,allocatable :: ifkp(:),jskp(:),kkb(:,:)
  integer,allocatable :: iccb(:),icptbl(:,:),indm(:)
  integer             :: npoint
end module axp

module enr
  implicit none
  integer,allocatable :: kxm(:,:),icm(:),iudm(:)
  character(2),allocatable :: mrrm(:)
  integer,allocatable :: mrnm(:),mweim(:),nstm(:)
  integer,allocatable :: neigm(:)
  real(8),allocatable :: eigm(:,:)
  real(8),allocatable :: eig(:)
  integer :: nkpint
end module enr
  
module mrk
  integer, allocatable :: nsit(:),MSIT(:,:)
  real(8), allocatable :: xims(:,:)
end module mrk

module canvas
!
! by H.Funashima,2010
!
  integer :: ifile
  integer :: ipoint
end module canvas


module check_array_size
  implicit none
  contains
  subroutine set_initial_params
    use ayband_param
    use axe, only:eax,nlin,jrr,jud
    use axp, only:ifkp,jskp,kkb,iccb,icptbl,indm
    use enr, only:kxm,icm,iudm,mrrm,mrnm,mweim,nstm,neigm,eigm,eig
    use mrk
    !
    ! for axe
    !
        allocate(eax(n65,maxeig,maxirp) )
        allocate( NLIN(maxirp), JRR(maxirp), JUD(maxirp) )
    !
    ! for axp
    !
        allocate( ifkp(maxfkp), jskp(maxfkp), kkb(3,maxfkp) )
        allocate( iccb(maxfkp), icptbl(12,maxfkp), indm(maxfkp) )
    !
    ! for enr
    !
        allocate( kxm(3,maxkpt),icm(maxkpt),iudm(maxkpt) )
        allocate( mrrm(maxkpt),mrnm(maxkpt),mweim(maxkpt),nstm(maxkpt) )
        allocate( neigm(maxkpt),eigm(maxeig,maxkpt) )
        allocate( eig(maxeig) )
    !
    ! for mrk
    !
        allocate(nsit(maxkpt),msit(3,maxkpt),xims(3,maxkpt))
    return
  end subroutine set_initial_params

  subroutine check_nkp(nkp)
    use ayband_param, only:maxkpt,maxeig
    use scl, only:ipr
    use enr, only:kxm,icm,iudm,mrrm,mrnm,mweim,nstm,neigm,eigm
    use mrk
    implicit none
    integer :: nkp
!
! for enr
!
    integer,allocatable :: kxm_bk(:,:),icm_bk(:),iudm_bk(:)
    character(2),allocatable :: mrrm_bk(:)
    integer,allocatable :: mrnm_bk(:),mweim_bk(:),nstm_bk(:)
    integer,allocatable :: neigm_bk(:)
    real(8),allocatable :: eigm_bk(:,:)
    integer, allocatable :: nsit_bk(:),msit_bk(:,:)
    real(8), allocatable :: xims_bk(:,:)
    integer :: i,j
    if( nkp > maxkpt ) then
        allocate( kxm_bk(3,maxkpt),icm_bk(maxkpt),iudm_bk(maxkpt) )
        allocate( mrrm_bk(maxkpt),mrnm_bk(maxkpt),mweim_bk(maxkpt),nstm_bk(maxkpt) )
        allocate( neigm_bk(maxkpt),eigm_bk(maxeig,maxkpt) )
        if ( ipr.ge.5) then
          write(*, '(3X,"re-allocate for nkp=",i6," in sub check_nkp (maxkpt=",i6," -->",i7,")")')nkp,maxkpt,maxkpt*2
        endif
        allocate(nsit_bk(maxkpt),msit_bk(3,maxkpt),xims_bk(3,maxkpt) )
        !
        ! copy to buffer
        !
        do j=1,maxkpt
        !
        ! for enr
        !
          do i=1,3
            kxm_bk(i,j)=kxm(i,j)
          enddo
          icm_bk(j)=icm(j)
          iudm_bk(j)=iudm(j)
          mrrm_bk(j)=mrrm(j)
          mrnm_bk(j)=mrnm(j)
          mweim_bk(j)=mweim(j)
          nstm_bk(j)=nstm(j)
          neigm_bk(j)=neigm(j)
          do i=1,maxeig
            eigm_bk(i,j)=eigm(i,j)
          enddo
        !
        ! for mrk
        !
          nsit_bk(j) = nsit(j)
          do i=1,3
            msit_bk(i,j)=msit(i,j)
            xims_bk(i,j)=xims(i,j)
          enddo
        enddo
        deallocate(kxm,icm,iudm,mrrm,mrnm,mweim,nstm,neigm,eigm)
        deallocate(nsit,msit,xims)
        maxkpt = maxkpt * 2
        !
        ! re-allocate
        !
        allocate( kxm(3,maxkpt),icm(maxkpt),iudm(maxkpt) )
        allocate( mrrm(maxkpt),mrnm(maxkpt),mweim(maxkpt),nstm(maxkpt) )
        allocate( neigm(maxkpt),eigm(maxeig,maxkpt) )
!
        allocate(nsit(maxkpt),msit(3,maxkpt),xims(3,maxkpt))
        do j=1, maxkpt / 2
          do i=1,3
            kxm(i,j)=kxm_bk(i,j)
          enddo
          icm(j)=icm_bk(j)
          iudm(j)=iudm_bk(j)
          mrrm(j)=mrrm_bk(j)
          mrnm(j)=mrnm_bk(j)
          mweim(j)=mweim_bk(j)
          nstm(j)=nstm_bk(j)
          neigm(j)=neigm_bk(j)
          do i=1,maxeig
            eigm(i,j)=eigm_bk(i,j)
          enddo
!
          nsit(j)=nsit_bk(j)
          do i=1,3
            msit(i,j)=msit_bk(i,j)
            xims(i,j)=xims_bk(i,j)
          enddo
        enddo
        deallocate(kxm_bk,icm_bk,iudm_bk,mrrm_bk,mrnm_bk,mweim_bk,nstm_bk,neigm_bk,eigm_bk)
        deallocate(nsit_bk,msit_bk,xims_bk)
    endif
  end subroutine check_nkp


  subroutine check_npoint
    use ayband_param, only:maxfkp
    use scl, only:ipr
    use axp
    implicit none
    integer,allocatable :: ifkp_bk(:),jskp_bk(:),kkb_bk(:,:)
    integer,allocatable :: iccb_bk(:),icptbl_bk(:,:),indm_bk(:)
    integer :: i,j
    if ( npoint > maxfkp ) then
        if ( ipr.ge.5) then
          write(*, '(3X,"re-allocate for npoint=",i6," in sub check_npoint (maxfkp=",i6," -->",i7,")")')npoint,maxfkp,maxfkp*2
        endif
        allocate(ifkp_bk(maxfkp),jskp_bk(maxfkp),kkb_bk(3,maxfkp))
        allocate(iccb_bk(maxfkp),icptbl_bk(12,maxfkp),indm_bk(maxfkp))
        !
        ! copy to buffer
        !
        do j=1,maxfkp
          ifkp_bk(j)=ifkp(j)
          jskp_bk(j)=jskp(j)
          do i=1,3
            kkb_bk(i,j)=kkb(i,j)
          enddo
          iccb_bk(j)=iccb(j)
          do i=1,12
            icptbl_bk(i,j)=icptbl(i,j)
          enddo
          indm_bk(j)=indm(j)
        enddo
        deallocate(ifkp,jskp,kkb,iccb,icptbl,indm)
        maxfkp = maxfkp * 2
        !
        ! re-allocate array
        !
        allocate(ifkp(maxfkp),jskp(maxfkp),kkb(3,maxfkp))
        allocate(iccb(maxfkp),icptbl(12,maxfkp),indm(maxfkp))
        do j=1,maxfkp / 2
          ifkp(j)=ifkp_bk(j)
          jskp(j)=jskp_bk(j)
          do i=1,3
            kkb(i,j)=kkb_bk(i,j)
          enddo
          iccb(j)=iccb_bk(j)
          do i=1,12
            icptbl(i,j)=icptbl_bk(i,j)
          enddo
          indm(j)=indm_bk(j)
        enddo
        deallocate(ifkp_bk,jskp_bk,kkb_bk,iccb_bk,icptbl_bk,indm_bk)
    endif
    return
  end subroutine check_npoint

  subroutine check_naxen
    use ayband_param, only:n65,maxeig,maxirp
    use scl, only:ipr
    use axe
    implicit none
    real(8),allocatable :: eax_bk(:,:,:)
    integer,allocatable :: NLIN_bk(:),JRR_bk(:),JUD_bk(:)
    integer :: i,j,k
    if ( naxen > maxirp ) then
        if ( ipr.ge.5) then
          write(*, '(3X,"re-allocate for naxen=",i6," in sub check_naxen (maxirp=",i6," -->",i7,")")')naxen,maxirp,maxirp*2
        endif
        allocate(eax_bk(n65,maxeig,maxirp) )
        allocate(NLIN_BK(maxirp), JRR_BK(maxirp), JUD_BK(maxirp) )
        !
        ! copy to buffer
        !
        do j=1,maxirp
          do i=1,maxeig
            do k=1,n65
              eax_bk(k,i,j) = eax(k,i,j)
            enddo
          enddo
          NLIN_bk(j)=NLIN(j)
          JRR_bk(j) =JRR(j)
          JUD_BK(j) =JUD(j)
        enddo
        deallocate(eax)
        deallocate(NLIN,JRR,JUD)
        maxirp = maxirp * 2
        !
        ! re-allocate array
        !
        allocate(eax(n65,maxeig,maxirp) )
        allocate(NLIN(maxirp), JRR(maxirp), JUD(maxirp) )
        do j=1, maxirp / 2
          do i=1,maxeig
            do k=1,n65 
              eax(k,i,j) = eax_bk(k,i,j)
            enddo
          enddo
          NLIN(j)=NLIN_bk(j)
          JRR(j) =JRR_bk(j)
          JUD(j) =JUD_bk(j)
        enddo
        deallocate(eax_bk)
        deallocate(NLIN_bk,JRR_BK,JUD_BK)
    endif
    return
  end subroutine check_naxen

  subroutine check_neig(neig)
    use ayband_param, only:maxeig,maxirp,maxkpt,n65
    use axe, only:eax
    use enr, only:eigm,eig
    use scl, only:ipr
    implicit none
    real(8),allocatable :: eax_bk(:,:,:)
    real(8),allocatable :: eigm_bk(:,:)
    real(8),allocatable :: eig_bk(:)
    integer :: neig
    integer :: i,j,k
    integer :: nw
!
    if ( neig > maxeig ) then
        nw = 2
        do while ( neig > nw * maxeig )
          nw = nw * 2
        enddo
        if ( ipr.ge.5) then
          write(*, '(3X,"re-allocate for neig=",i6," in sub check_neig (maxeig=",i6," -->",i7,")")')neig,maxeig,maxeig*2
        endif

        allocate(eax_bk(n65,maxeig,maxirp) )
        allocate(eigm_bk(maxeig,maxkpt) )
        allocate(eig_bk(maxeig) )
        !
        ! copy to buffer
        !
        do j=1,maxirp
          do i=1,maxeig
            do k=1,n65
              eax_bk(k,i,j) = eax(k,i,j)
            enddo
          enddo
        enddo
        do j=1,maxkpt
          do i=1,maxeig
            eigm_bk(i,j)=eigm(i,j)
          enddo
        enddo
        do j=1,maxeig
          eig_bk(j)=eig(j)
        enddo
        deallocate(eax)
        deallocate(eigm)
        deallocate(eig)
        maxeig = maxeig * nw
        !
        ! re-allocate array
        !
        allocate(eax(n65,maxeig,maxirp) )
        allocate(eigm(maxeig,maxkpt) )
        allocate(eig(maxeig) )
        do j=1, maxirp 
          do i=1,maxeig / nw
            do k=1,n65 
              eax(k,i,j) = eax_bk(k,i,j)
            enddo
          enddo
        enddo
        do j=1,maxkpt
          do i=1,maxeig / nw
            eigm(i,j) = eigm_bk(i,j)
          enddo
        enddo
        do j=1,maxeig / nw
            eig(j)=eig_bk(j)
        enddo
       deallocate(eax_bk)
       deallocate(eigm_bk)
       deallocate(eig_bk)
    endif
   return
  end subroutine check_neig

end module check_array_size
